#ifndef OPTION_ASDL_H
#define OPTION_ASDL_H

namespace option_asdl {

#define ASDL_NAMES struct
ASDL_NAMES option_i {
  enum no_name {
  errexit = 1,
  nounset = 2,
  pipefail = 3,
  inherit_errexit = 4,
  nullglob = 5,
  verbose_errexit = 6,
  noexec = 7,
  xtrace = 8,
  verbose = 9,
  noglob = 10,
  noclobber = 11,
  posix = 12,
  vi = 13,
  emacs = 14,
  interactive = 15,
  hashall = 16,
  failglob = 17,
  extglob = 18,
  nocasematch = 19,
  eval_unsafe_arith = 20,
  _allow_command_sub = 21,
  _allow_process_sub = 22,
  dynamic_scope = 23,
  redefine_module = 24,
  _running_trap = 25,
  _running_hay = 26,
  _no_debug_trap = 27,
  _no_err_trap = 28,
  strict_argv = 29,
  strict_arith = 30,
  strict_array = 31,
  strict_control_flow = 32,
  strict_errexit = 33,
  strict_nameref = 34,
  strict_word_eval = 35,
  strict_tilde = 36,
  strict_glob = 37,
  parse_at = 38,
  parse_proc = 39,
  parse_func = 40,
  parse_brace = 41,
  parse_bracket = 42,
  parse_equals = 43,
  parse_paren = 44,
  parse_ysh_string = 45,
  parse_triple_quote = 46,
  simple_word_eval = 47,
  dashglob = 48,
  command_sub_errexit = 49,
  process_sub_fail = 50,
  xtrace_rich = 51,
  xtrace_details = 52,
  sigpipe_status_ok = 53,
  redefine_proc_func = 54,
  parse_at_all = 55,
  parse_backslash = 56,
  parse_backticks = 57,
  parse_dollar = 58,
  parse_ignored = 59,
  parse_sh_arith = 60,
  parse_dparen = 61,
  parse_dbracket = 62,
  parse_bare_word = 63,
  simple_echo = 64,
  simple_eval_builtin = 65,
  simple_test_builtin = 66,
  expand_aliases = 67,
  lastpipe = 68,
  progcomp = 69,
  histappend = 70,
  hostcomplete = 71,
  cmdhist = 72,
  assoc_expand_once = 73,
  autocd = 74,
  cdable_vars = 75,
  cdspell = 76,
  checkhash = 77,
  checkjobs = 78,
  checkwinsize = 79,
  complete_fullquote = 80,
  direxpand = 81,
  dirspell = 82,
  dotglob = 83,
  execfail = 84,
  extdebug = 85,
  extquote = 86,
  force_fignore = 87,
  globasciiranges = 88,
  globstar = 89,
  gnu_errfmt = 90,
  histreedit = 91,
  histverify = 92,
  huponexit = 93,
  interactive_comments = 94,
  lithist = 95,
  localvar_inherit = 96,
  localvar_unset = 97,
  login_shell = 98,
  mailwarn = 99,
  no_empty_cmd_completion = 100,
  nocaseglob = 101,
  progcomp_alias = 102,
  promptvars = 103,
  restricted_shell = 104,
  shift_verbose = 105,
  sourcepath = 106,
  xpg_echo = 107,
  ARRAY_SIZE = 108,
  };
};

typedef int option_t;

ASDL_NAMES builtin_i {
  enum no_name {
  colon = 1,
  dot = 2,
  exec_ = 3,
  eval = 4,
  set = 5,
  shift = 6,
  times = 7,
  trap = 8,
  unset = 9,
  builtin = 10,
  readonly = 11,
  local = 12,
  declare = 13,
  typeset = 14,
  export_ = 15,
  true_ = 16,
  false_ = 17,
  try_ = 18,
  read = 19,
  echo = 20,
  printf = 21,
  mapfile = 22,
  readarray = 23,
  cd = 24,
  pushd = 25,
  popd = 26,
  dirs = 27,
  pwd = 28,
  source = 29,
  umask = 30,
  ulimit = 31,
  wait = 32,
  jobs = 33,
  fg = 34,
  bg = 35,
  shopt = 36,
  complete = 37,
  compgen = 38,
  compopt = 39,
  compadjust = 40,
  compexport = 41,
  getopts = 42,
  command = 43,
  type = 44,
  hash = 45,
  help = 46,
  history = 47,
  alias = 48,
  unalias = 49,
  bind = 50,
  append = 51,
  write = 52,
  json = 53,
  json8 = 54,
  pp = 55,
  hay = 56,
  haynode = 57,
  module = 58,
  use = 59,
  error = 60,
  failed = 61,
  fork = 62,
  forkwait = 63,
  fopen = 64,
  shvar = 65,
  ctx = 66,
  runproc = 67,
  boolstatus = 68,
  test = 69,
  bracket = 70,
  push_registers = 71,
  is_main = 72,
  cat = 73,
  ARRAY_SIZE = 74,
  };
};

typedef int builtin_t;


}  // namespace option_asdl

#endif  // OPTION_ASDL_H
